/* $Id$ */
#ifndef SPLICEENDSGEN_INCLUDED
#define SPLICEENDSGEN_INCLUDED

typedef struct Spliceendsgen_T *Spliceendsgen_T;

#include "spliceends.h"

#define T Spliceendsgen_T

extern T
Spliceendsgen_new ();
extern void
Spliceendsgen_reset (T this);
extern void
Spliceendsgen_free_memory (T this);
extern void
Spliceendsgen_free (T *old);
extern Spliceends_T
Spliceendsgen_checkout (T this, int querylength);
extern void
Spliceendsgen_return (T this, Spliceends_T *old);

#undef T
#endif



